#include <u.h>
#include <libc.h>
#include <bio.h>
#include <json.h>

int Dumpvalfail;

static void map(Biobuf *bp, int indent, char *buf, int len, Jmap *m);
static void array(Biobuf *bp, int indent, char *buf, int len, Jarray *a);


static void
value(Biobuf *bp, int indent, char *buf, int len, Jvalue *v)
{
	if(v == nil){
		Bprint(bp, "### value==nil ###");
		return;
	}

	switch(v->type){
	case Tnull:
		Bprint(bp, "%s=null\n", buf);
		break;
	case Ttrue:
		Bprint(bp, "%s=true\n", buf);
		break;
	case Tfalse:
		Bprint(bp, "%s=false\n", buf);
		break;
	case Tnumber:
		if(floor(v->num) == v->num)
			Bprint(bp, "%s=%.0f\n", buf, v->num);
		else
			Bprint(bp, "%s=%g\n", buf, v->num);
		break;
	case Tstring:
		Bprint(bp, "%s=%q\n", buf, v->str);
		break;
	case Tmap:
		map(bp, indent, buf, len, v->map);
		break;
	case Tarray:
		array(bp, indent, buf, len, v->array);
		break;
	case Tempty:
		break;
	default:
		sysfatal("value: %d unknown value type", v->type);
		break;
	}
}

static void
array(Biobuf *bp, int indent, char *buf, int len, Jarray *a)
{
	int i, got;

	got = strlen(buf);
	for(i = 0; a && i < a->used; i++){
		snprint(buf+got, len-got, "[%d]", i);
		value(bp, indent+1, buf, len, a->base[i]);
	}
}

static void
map(Biobuf *bp, int indent, char *buf, int len, Jmap *map)
{
	int got;
	Jmap *m;

	got = strlen(buf);
	for(m = map; m; m = m->next){
		snprint(buf+got, len-got, "%s%q", ((indent != 0)? ".": ""), m->name);
		value(bp, indent+1, buf, len, m->value);
	}
}

void
json_gron(Biobuf *bp, Jvalue *v)
{
	char buf[8192];

	*buf = 0;

	switch(v->type){
	case Tmap:
		map(bp, 0, buf, sizeof(buf), v->map);
		break;
	case Tarray:
		array(bp, 0, buf, sizeof(buf), v->array);
		break;
	default:
		sysfatal("json_gron: %d unknown object type\n", v->type);
		break;
	}
}

static void
usage(void)
{
	fprint(2, "usage: %s [file]\n", argv0);
	exits("usage");
}

void
main(int argc, char *argv[])
{
	Jvalue *v;
	Biobuf out, *bp;

	ARGBEGIN{
		default:
			usage();
	}ARGEND;

	if(argc == 0){
		if((v = json_parse(0, "stdin")) == nil)
			sysfatal("stdin: %r\n");
	}
	else{
		if((bp = Bopen(argv[0], OREAD)) == nil)
			sysfatal("%s cannot open\n", argv[0]);
		if((v = json_Bparse(bp, argv[0])) == nil)
			sysfatal("%s: %r\n", argv[0]);
		Bterm(bp);
	}

	quotefmtinstall();

	Binit(&out, 1, OWRITE);
	json_gron(&out, v);
	Bterm(&out);

	exits(nil);
}
